$def with (stats)

<div id='stats_panel'>

<img src="$base/static/images/16/connections.png" title="$_('Connections')">$stats.num_connections ($deluge_int(stats.max_num_connections))

<img src="$base/pixmaps/downloading16.png" title="$_('Down Speed')">$fspeed(stats.payload_download_rate)  ($deluge_int(stats.max_download))

<img src="$base/pixmaps/seeding16.png" title="$_('Up Speed')">$fspeed(stats.payload_upload_rate)  ($deluge_int(stats.max_upload))

<img src="$base/pixmaps/traffic16.png" title="$_('Overhead')">$fspeed(stats.download_rate - stats.payload_download_rate),$fspeed(stats.upload_rate - stats.payload_upload_rate)

<img src="$base/pixmaps/dht16.png" title="$_('DHT Nodes')">$stats.dht_nodes

<img src="$base/static/images/16/drive-harddisk.png" title="$_('Disk Space')">$fsize(stats.free_space)

$if stats.has_incoming_connections:
$else:
    <img src="$base/pixmaps/alert16.png" title="$_('No Incoming Connections')">


</div>



