<dl>
	<dt class="downloaded">$_('Downloaded'):</dt><dd class="downloaded"/>
	<dt class="uploaded">$_('Uploaded'):</dt><dd class="uploaded"/>
	<dt class="share">$_('Share Ratio'):</dt><dd class="share"/>
	<dt class="announce">$_('Next Announce'):</dt><dd class="announce"/>
	<dt class="tracker">$_('Tracker Status'):</dt><dd class="tracker_status"/>
</dl>
<dl>
	<dt class="downspeed">$_('Speed'):</dt><dd class="downspeed"/>
	<dt class="upspeed">$_('Speed'):</dt><dd class="upspeed"/>
	<dt class="eta">$_('ETA'):</dt><dd class="eta"/>
	<dt class="pieces">$_('Pieces'):</dt><dd class="pieces"/>
</dl>
<dl>
	<dt class="seeders">$_('Seeders'):</dt><dd class="seeders"/>
	<dt class="peers">$_('Peers'):</dt><dd class="peers"/>
	<dt class="avail">$_('Availability'):</dt><dd class="avail"/>
	<dt class="auto_managed">$_('Auto Managed'):</dt><dd class="auto_managed"/>
</dl>
<dl>
	<dt class="active_time">$_('Active Time'):</dt><dd class="active_time"/>
	<dt class="seeding_time">$_('Seeding Time'):</dt><dd class="seeding_time"/>
	<dt class="seed_rank">$_('Seed Rank'):</dt><dd class="seed_rank"/>
</dl>
<br style="clear: both;" />
