$for id, title, image, flag, method, url, important in toolbar_items:
    $if important:
        <a class='toolbar_btn' href="#"
        onclick='toolbar_$(method.lower()) ("$base$url",$flag)'
        title='$title'><img class='toolbar_btn'
        src='$base/static/images/16/$image'></a>

        <a class='toolbar_btn' href="#"
        onclick='select_all_rows()'
        title='$_("Select All")'><img class='toolbar_btn'
        src='$base/static/images/16/select-all.png'></a>


