<div id="preferences">
	<div class="categories">
		<h3>$_('Categories')</h3>
		<ul>
		</ul>
	</div>
	<div class="pref_pages">
		<h3></h3>
	</div>
	<div class="buttons">
		<button class="reset deluge_button">$_('Reset')</button><button class="apply deluge_button">$_('Apply')</button>
	</div>
</div>
