$def with (torrent)
<table width="100%"><tr>
<td colspan=4 style="background-color:#999;-moz-border-radius:5px;">

    <div class="progress_bar progress_bar_detail" style="width:$torrent.progress%;">

    $("%.2f" % torrent.progress) %</div>
</td>
</tr><td width=25%%>
<table>

<tr><td class="info_label">$_('Downloaded'):</td>
<td class="info_value">$fsize(torrent.total_done)</td></tr>

<tr><td class="info_label">$_('Uploaded'):</td>
<td class="info_value">$fsize(torrent.total_payload_upload)</td>
</tr>



<tr><td class="info_label">$_('Share Ratio'):</td>
<td class="info_value">$("%.3f" % torrent.ratio)</td></tr>

<tr><td class="info_label">$_('Next Announce'):</td>
<td class="info_value">$torrent.next_announce </td></tr>




</table>
</td><td width=25%>

<table>
<tr><td class="info_label">$_('Speed'):</td><td class="info_value">
$fspeed(torrent.download_payload_rate)</td></td></tr>

<tr><td class="info_label">$_('Speed'):</td>
<td class="info_value">$fspeed(torrent.upload_payload_rate)</td></tr>


<tr><td class="info_label">$_('ETA'):</td>
<td class="info_value">$ftime(torrent.eta) </td></tr>

<tr><td class="info_label">$_('Pieces'):</td>
<td class="info_value">$torrent.num_pieces ($fsize(torrent.piece_length)) </td>
</tr>

</table>


</td><td width=25%%>

<table>
<tr><td class="info_label">$_('Seeders'):</td>
<td class="info_value">$torrent.num_seeds  ($torrent.total_seeds )</td></tr>

<tr><td class="info_label">$_('Peers'):</td>
<td class="info_value">$torrent.num_peers  ($torrent.total_peers )</td></tr>

<tr><td class="info_label">$_('Availability'):</td>
<td class="info_value">$("%.3f" % torrent.distributed_copies)</td></td></tr>



<tr><td class="info_label">$_('Auto Managed'):</td>
<td class="info_value">
$if torrent.is_auto_managed:
    $_('True')
$else:
    $_('False')
</td></tr>


</table>

</td><td width=25%%>
<table>

<tr><td class="info_label">$_('Active time'):</td>
    <td class="info_value">$ftime(torrent.active_time)</td>
</tr>
<tr><td class="info_label">$_('Seeding time'):</td>
    <td class="info_value">$ftime(torrent.seeding_time)</td>
</tr>
<tr><td class="info_label">$_('Seed rank'):</td>
    <td class="info_value">$(torrent.seed_rank)</td>
</tr>

<tr><td class="info_label">$_('Queue Position'):</td>
<td class="info_value">
$if (torrent.queue == -1):
    &#8734; <!--infinite symbol-->
$else:
    $(torrent.queue + 1)
</td></tr>

</table></td>


</table>


