$def with (column_id, column_name, order, active_up, active_down)
<th class="torrent_table">
<a href="$base/index?sort=$column_id&order=$order&filter_cat=$get('filter_cat')&filter_value=$get('filter_value')"">
$column_name\
$if active_up:
    <img src="$base/static/images/16/up.png" />
$if active_down:
    <img src="$base/static/images/16/down.png" />
</a>
</th>


