$def with (torrent)
<table width="100%">
<!--long values-->
<!--<tr><td>&nbsp;</td></tr>-->
<tr colspan="3">
    <table>
    <tr><td class="info_label">$('Name'):</td>
        <td class="info_value">$torrent.name</td>
    </tr>
    <tr><td class="info_label">$('Hash'):</td>
        <td class="info_value">$torrent.id</td>
    </tr>

    <tr>
    <td class="info_label"><a href="/torrent/move/$torrent.id" target="_top">$('Path'):</td>
        <td class="info_value" title="$torrent.save_path">$crop_left(torrent.save_path, 200)</td>
    </tr>

    <tr>
    <td class="info_label">$_('Total Size'):</td>
    <td class="info_value">$fsize(torrent.total_size)</td>
    </tr>

    <tr>
    <td class="info_label">$_('# Of Files'):</td>
    <td class="info_value">$torrent.num_files</td>
    </tr>

    <tr><td class="info_label">$_('Tracker Status'):</td>
        <td class="info_value" title="$torrent.tracker_status">$(crop(torrent.tracker_status, 200)) </td>
    </tr>

    <tr><td class="info_label">$_('Tracker'):</td>
        <td class="info_value" title="$torrent.tracker">$(crop(torrent.tracker, 200))</td>
    </tr>
    <tr>
    <td class="info_label">$_('Private'):</td>
        <td class="info_value">$torrent.private</td>
    </tr>


    </table>
</tr>
</table>
