<form id="bandwidth_preferences">
	<fieldset>
		<legend>$_('Global Bandwidth Usage')</legend>
		<label>$_('Max Connections'):</label>
		<input type="text" name="max_connections_global" rel="spinner" />
		<br/>
		
		<label>$_('Max Upload Slots'):</label>
		<input type="text" name="max_upload_slots_global" rel="spinner" />
		<br/>
		
		<label>$_('Max Download Speed') (KiB/s):</label>
		<input type="text" name="max_download_speed" rel="spinner1" />
		<br/>
		
		<label>$_('Max Upload Speed') (KiB/s):</label>
		<input type="text" name="max_upload_speed" rel="spinner1" />
		<br/>
		
		<label>$_('Max Half-Open Connections'):</label>
		<input type="text" name="max_half_open_connections" rel="spinner" />
		<br/>
		
		<label>$_('Max Conection Attempts per Second'):</label>
		<input type="text" name="max_connections_per_second" rel="spinner" />
		<br/>
		
		<label class="fluid">
			<input type="checkbox" name="ignore_limits_on_local_network"/>
			$_('Ignore limits on local network')
		</label><br/>
	</fieldset>
	<fieldset>
		<legend>$_('Per Torrent Bandwidth Usage')</legend>
		<label>$_('Max Connections'):</label>
		<input type="text" name="max_connections_per_torrent" rel="spinner" />
		<br/>
		
		<label>$_('Max Upload Slots'):</label>
		<input type="text" name="max_upload_slots_per_torrent" rel="spinner" />
		<br/>
		
		<label>$_('Max Download Speed') (KiB/s):</label>
		<input type="text" name="max_download_speed_per_torrent" rel="spinner1" />
		<br/>
		
		<label>$_('Max Upload Speed') (KiB/s):</label>
		<input type="text" name="max_upload_speed_per_torrent" rel="spinner1" />
		<br/>
	</fieldset>
</form>
