<div id="files">
	<table>
		<thead>
			<tr class="header">
				<th class="filename">$_('Filename')</th>
				<th class="size">$_('Size')</th>
				<th class="progress">$_('Progress')</th>
				<th class="priority">$_('Priority')</th>
			</tr>
		</thead>
		<tbody/>
	</table>
</div>
