<fieldset>
    <label>Piece Size:</label>
    <select>
        <option>32 KiB</option>
        <option>64 KiB</option>
        <option>128 KiB</option>
        <option>256 KiB</option>
        <option>512 KiB</option>
    </select>
    <br/>
    
    <label class="fluid">
        <input type="checkbox" />
        $_('Set Private Flag')
    </label><br/>
    
    <label class="fluid">
        <input type="checkbox" />
        $_('Add this torrent to the session')
    </label>
</fieldset>
