$:render.header(_('Set Timeout'))
<div class="panel">
<form action="$base/refresh/set" method="POST">
    $_('Refresh page every:')
    <input type="text" name="refresh" value="$getcookie('auto_refresh_secs')"
    size="3">
    $_('seconds')
    <input type="submit" value="$_('Set')">
</form>
</div>
$:render.footer()
