/*

-----------------------------------------------------------

Theme Name: Ajax White
Theme URI: http://deluge-torrent.org
Description: Deluge Ajax Theme
Version: 0.1
-----------------------------------------------------------

*/

body {
	background: #ffffff;
	font-family: trebuchet ms;
	font-size: 0.7em;
}

#toolbar #buttons li:hover {
	background-color: #C3D9FF;
}

#toolbar {
	border-bottom: 1px solid Black;
	height: 40px;
}

#toolbar ul, #status ul {
	margin: 0;
	padding: 0;
	display: inline;
	list-style: none;
}

#toolbar ul li, #status ul li {
	float: left;
	display: block;
	margin: 2px;
	background-repeat: no-repeat;
	background-position: 2px;
}

#toolbar #buttons {
	float: left;
	background: url('/static/images/deluge32.png') no-repeat 2px;
	height: 40px;
	padding-left: 140px;
	width: 320px;
}

#toolbar #buttons li {
	width: 36px;
	height: 36px;
	-moz-border-radius:5px;
	-webkit-border-radius: 5px;
	cursor: pointer;
}

#toolbar #buttons li:hover {
	background-color: #E0ECFF;
}

.info { display: none; }

.title {
	position: absolute;
	left: 40px;
	top: 7px;
	color: #8fa6c3;
	padding: 0;
	margin: 0;
	line-height: 26px;
	font-size: 26px;
}

.moouiSplitter {
	background: none;
}

#labels {
	width: 160px;
	margin: 5px;
	overflow: auto;
	background: #b5edbc;
	-moz-border-radius: 5px;
}

#labels div {
    background: White;
	-moz-border-radius: 5px;
	-webkit-border-radius: 5px;
	margin: 5px;
	padding-top: 5px;
	padding-bottom: 20px;
}

#labels div h3 {
    margin: 5px;
}

#labels div ul {
	list-style: none;
	margin: 0;
	padding: 0;
}

.moouiDataGrid {
	overflow: auto;
	margin: 5px;
	border: 4px solid #C3D9FF;
	-moz-border-radius: 5px;
	-webkit-border-radius: 5px;
}

.moouiDataGrid table {
	/*background: #304663;*/
	border-spacing: 0px;
}

.moouiDataGrid table th {
	font-weight: normal;
	background: #C3D9FF;
	margin-bottom: 2px;
	cursor: pointer;
}

.moouiDataGrid table td {
	padding: 2px;
	border-bottom: 1px solid #1c2431;
	cursor: pointer;
}
.moouiDataGrid table tr.selected td { background: #E0ECFF; }
.moouiDataGrid table tr:hover td { background: #FFFFCC; }

.moouiProgressBar div {
	background: #C3D9FF;
	-moz-border-radius:5px; /*ff only setting*/
	-webkit-border-radius: 5px;
}

.moouiProgressBar span {
	text-align: center;
}

#details {
	padding: 5px;
}

#details .moouiTabs li {
	background: #E0ECFF;
	padding: 7px;
	font-size: 1.1em;
}

#details .moouiTabs li:hover {
	background: #C3D9FF;
}

#details li.moouiTabActive {
	background: #C3D9FF;
}

#details .moouiTabPage {
	background: #C3D9FF;
	padding: 5px;
	-moz-border-radius:0 5px 5px 5px;
	-webkit-border-radius: 0 5px 5px 5px;
	overflow: auto;
}

#details dl {
	float: left;
	min-height: 100px;
}

#details dl dt {
	float: left;
	font-weight: bold;
	height: 19px;
}

#details dl dd {
	margin-left: 100px;
	width: 140px;
	height: 19px;
}

#details dd.downspeed, #details dd.upspeed, #details dd.eta, #details dd.pieces {
	margin-left: 75px;
}

#details form br {
	clear: left;
}

#details dd.torrent_name {
	width: 500px;
}

#details dd.tracker {
	width: 500px;
}

#status {
	height: 40px;
	background: #E0ECFF;
	-moz-border-radius: 7px 7px 0px 0px;
}

.moouiMenu {
	background: #E0ECFF;
	border: 1px outset Gray;
}

ul.moouiMenu li.moouiMenuSep {
	background-color: Gray;
}

ul.moouiMenu li:hover {
	background-color: #a0a0a0;
	-moz-border-radius: 3px;
	-webkit-border-radius: 3px;
}

ul.moouiMenu li.moouiMenuSep:hover {
	background-color: Gray;
}

.moouiWindow {
	background: #C3D9FF;
	border: 1px solid #1c2431;
	-moz-border-radius: 5px;
	-webkit-border-radius: 5px;
}

.moouiWindowClose {
	background-image: url(/template/static/themes/white/window_close.png);
}

.moouiWindowTitle {
	margin: 5px;
}

/* forms */
form fieldset {
	border: none;
	float: left;
}

form legend {
	font-weight: bold;
}

form label {
	width: 150px;
    line-height: 20px;
	float: left;
}

form input, form select {
	float: left;
	margin-right: 5px;
	border:1px solid #23344b;
	background: #E0ECFF;
	color: #000;
	line-height: 20px;
}

form textarea {
	border:1px solid #23344b;
	background: #E0ECFF;
	width:480px;
}

form .disabled {
	color: Gray;
}

form .deluge_button, button.deluge_button  {
	background-color: #E0ECFF;
	border:1px solid #68a;
	cursor: pointer;
	background: #E0ECFF;
	color: #000;
	vertical-align:middle;
	-moz-border-radius:7px;
    -webkit-border-radius:7px;
	margin: 2px;
}

form .deluge_button:hover, button.deluge_button:hover {
	background-color:#68a;
}

form br {
	clear: left;
}

#addFromUrl label {
	width: 100px;
}

#addFromUrl input[type=submit] {
	margin: 5px;
	margin-left: 100px;
}

.moouiWindowContent #preferences .categories li:hover {
	text-decoration: underline;
}

#preferences #bandwidth_preferences label {
	width: 200px;
}

#preferences #encryption label {
	width: 70px;
}

#preferences #encryption label.fluid {
	width: auto;
}

#preferences #encryption select {
	width: 70px;
}

.deluge-prefs-active {
	font-weight: bold;
}

.moouiSpinner {
	width: 40px;
	margin-right: 20px;
}

label.fluid {
	width: auto;
	margin-right: 30px;
}

.deluge-prefs-page {
	overflow: auto;
}


#addTorrent select {
    width: 520px;
    height: 95px;
    margin: 10px;
    background: #99acc3;
    border:1px solid #23344b;
    color: Black;
}

#addTorrent #addButtons {
    width: 240px;
    margin: 0 auto;
}

#addTorrent #tabs .moouiTabPage {
    height: 200px;
}

#addTorrent .filesTable {
    margin: 10px;
    width: 510px;
    height: 180px;
    color: Black;
    background: #99acc3;
    border:1px solid #23344b;
}

#addTorrent #buttons {
    margin-top: 10px;
    float: right;
}

#createTorrent h3 {
    margin-bottom: 0px;
}

#createTorrentTabs .moouiTabPage {
    height: 110px;
}

#createTorrentTabs textarea {
    width: 360px;
    height: 95px;
    margin: 0;
}

#fileButton #folderButton {
    margin: 0 auto 10px auto;
    text-align: center;
}

#createTorrentFiles {
    height: 90px;
    width: 370px;
    margin: 10px;
    border:1px solid #23344b;
    background: #99acc3;
}

#createTorrentTrackersTable {
    margin: 5px;
    float: left;
    width: 280px;
    height: 95px;
    background: #99acc3;
    border:1px solid #23344b;
    color: Black;
}

#createTorrentTrackersTable thead tr {
    background: #304663 url(/static/images/simple_bg_flipped.jpg) repeat-x;
    color: White;
}

#createTorrentTabs .tier {
    width: 70px;
}

#createTorrentTabs .tracker {
    width: 230px;
}

#createTorrentTrackersButtons {
    float: right;
    width: 80px;
}

#createTorrentTrackersButtons button {
    width: 70px;
}

#createButtons {
    float: right;
}
