<form>
	<fieldset>
		<legend>$_('Port')</legend>
		<label>$_('Daemon Port'):</label><input type="text" name="daemon_port" rel="spinner" /><br/>
	</fieldset>
	<fieldset>
		<legend>$_('Connections')</legend>
		<label class="fluid"><input type="checkbox" name="allow_remote" />$_('Allow Remote Connections')</label>
	</fieldset>
	<fieldset>
		<legend>$_('Other')</legend>
		<label class="fluid"><input type="checkbox" name="new_release_check" />$_('Periodically check the website for new releases')</label>
	</fieldset>
</form>
