<div id="peers" class="gridview">
	<table>
		<thead>
			<tr class="header">
				<th class="country"></th>
				<th class="name">$_('Address')</th>
				<th class="client">$_('Client')</th>
				<th class="downspeed">$_('Down Speed')</th>
				<th class="upspeed">$_('Up Speed')</th>
			</tr>
		</thead>
		<tbody/>
	</table>
</div>
