$def with (title, active_tab="NONE")
$:render.basic_header(title)

<table width="100%"><tr>
<td>
    <div id="home_top">
    <img src="$base/static/images/deluge32.png" style="position:absolute;top:5px;"\>
    &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
    &nbsp;&nbsp;
    <a
    $if 'home' == active_tab:
         class="tab_button_active"
    $else:
        class="tab_button"
    href='$base/home'>Home</a>

<!--ADMIN-TABS-->
$for id, title, url in admin_pages:
    <a
    $if id == active_tab:
         class="tab_button_active"
    $else:
        class="tab_button"
    href='$base$url'>$title</a>
<a href="http://dev.deluge-torrent.org/wiki/Faq">Faq</a>
</div>

</td><td align="left">
$if is_auto_refreshed:
    $:render.part_auto_refresh()
</td><td>

</td><td align="right">
    $:part_stats()
</td></tr></table>

<hr style="margin-left:20px;">
