<div id="createTorrentTrackersTable">
    <table>
        <thead>
            <tr>
                <th class="tier">Tier</th>
                <th class="tracker">Tracker</th>
            </tr>
        </thead>
        <tbody />
    </table>
</div>
<div id="createTorrentTrackersButtons">
    <button class="delugeButton">$_('Up')</button>
    <button class="delugeButton">$_('Add')</button>
    <button class="delugeButton">$_('Remove')</button>
    <button class="delugeButton">$_('Down')</button>
</div>
