/*

-----------------------------------------------------------

Theme Name: Ajax Classic
Theme URI: http://deluge-torrent.org
Description: Deluge Ajax Theme
Version: 0.1
-----------------------------------------------------------

*/

body {
	background: #304663 url(/static/images/simple_bg.jpg) repeat-x;
	font-family: trebuchet ms;
	font-size: 0.7em;
	color: White;
}

iframe {
    border: none;
}

#toolbar {
	background: url(/static/images/simple_line.jpg) repeat-x 0px 69px;
	height: 75px;
}

#toolbar ul, #status ul {
	margin: 0;
	padding: 0;
	display: inline;
	list-style: none;
}

#toolbar ul li, #status ul li {
	float: left;
	display: block;
	margin: 2px;
	background-repeat: no-repeat;
	background-position: 2px;
}

#toolbar #buttons {
	float: left;
	background: url('/static/images/simple_logo.jpg') no-repeat;
	height: 71px;
	padding-left: 60px;
	width: 360px;
}

#toolbar #buttons li {
	width: 36px;
	height: 36px;
	-moz-border-radius:5px;
	-webkit-border-radius: 5px;
	cursor: pointer;
	margin-top: 30px;
}

#toolbar #buttons li:hover {
	background-color: #386594;
}

.info {
	text-align: right;
	padding: 0 50px 0 0;
	color: #8fa6c3;
	font-size: 16px;
	letter-spacing: 4px;
	font-weight: bold;
}

.title {
	color: #dce4ee;
	font-size: 32px;
	padding: 10px 50px 0 0;
	text-align: right;
}

.moouiSplitter {
	background: none;
}

.moouiSplitterVertical {
	background: url(split_vertical.png) repeat-y;
}

.moouiSplitterHorizontal {
	background: url(split_horizontal.png) repeat-x;
}

#labels {
	width: 140px;
	margin: 5px;
	overflow: auto;
}

#labels ul {
	list-style: none;
	margin: 10px;
	padding: 0;
}

.moouiDataGrid {
	overflow: auto;
	margin: 5px;
	border: 1px solid #2a425c;
}

.moouiDataGrid table {
	/*background: #304663;*/
	border-spacing: 0px;
}

.moouiDataGrid table th {
	font-weight: normal;
	border: 1px outset #1c2431;
	margin-bottom: 2px;
	cursor: pointer;
}

.moouiDataGrid table td {
	padding: 2px;
	border-bottom: 1px solid #1c2431;
	cursor: pointer;
}
.moouiDataGrid table tr.selected td { background: #900; }
.moouiDataGrid table tr:hover td { background: #68a; }

.moouiProgressBar div {
	background: #4573a5;
	-moz-border-radius:5px; /*ff only setting*/
	-webkit-border-radius: 5px;
}

.moouiProgressBar span {
	text-align: center;
}

#status {
	height: 40px;
	background: #1c2431 url(simple_line_flipped.png) repeat-x;
}

.moouiMenu {
	background: #304663 url(menu_bg.jpg) repeat-x;
	border: 1px outset #37506f;
}

ul.moouiMenu li.moouiMenuSep {
	background-color: #aaa;
}

ul.moouiMenu li:hover {
	background-color: #426187;
	-moz-border-radius: 3px;
	-webkit-border-radius: 3px;
}

ul.moouiMenu li.moouiMenuSep:hover {
	background-color: #aaa;
}

.moouiWindow {
	background: #37506f;
	border: 1px solid #1c2431;
}

.moouiWindowTitlebar {
    cursor: pointer;
	height: 30px;
	background: #1c2431 url(/static/images/simple_line.jpg) repeat-x 0px 25px;
}

.moouiWindowClose {
	background-image: url(window_close.png);
}

.moouiWindowTitle {
	margin: 5px;
}

/* forms */
form fieldset {
	border: none;
	float: left;
}

form legend {
	font-weight: bold;
}

form label {
	width: 150px;
	line-height: 20px;
	float: left;
}

form input, form select {
	float: left;
	margin-right: 5px;
	border:1px solid #23344b;
	background: #99acc3;
	color: #000;
	line-height: 20px;
}

form textarea {
	border:1px solid #23344b;
	background: #99acc3;
	width:480px;
}

form .disabled {
	color: Gray;
}

form .deluge_button, button  {
	background-color: #37506f;
	border:1px solid #68a;
	cursor: pointer;
	background: #99acc3;
	color: #000;
	vertical-align:middle;
	-moz-border-radius:7px;
	-webkit-border-radius:7px;
	margin: 2px;
}

form .deluge_button:hover, button:hover {
	background-color:#68a;
}

form br {
	clear: left;
}

#addFromUrl label {
	width: 100px;
}

#addFromUrl #url {
	width: 250px;
}

#addFromUrl .deluge_button {
	margin: 5px;
	margin-left: 100px;
}

.moouiWindowContent #preferences .categories li:hover {
	text-decoration: underline;
}

#preferences #bandwidth_preferences label {
	width: 200px;
}

#preferences #encryption label {
	width: 70px;
}

#preferences #encryption label.fluid {
	width: auto;
}

#preferences #encryption select {
	width: 70px;
}

.deluge-prefs-active {
	font-weight: bold;
}

label.fluid {
	width: auto;
	margin-right: 30px;
}

.deluge-prefs-page {
	overflow: auto;
}

.moouiSpinner {
    float: left;
	width: 66px;
	overflow: hidden;
}

.moouiSpinner input {
    float: left;
    border: 1px solid Black;
    width: 40px;
    margin: 0;
}

.moouiSpinner div {
    float: left;
}

.moouiSpinnerButtons {
    width: 9px;
    height: 10px;
    font-size: 0px;
}

.moouiSpinnerUp {
    cursor: pointer;
    background: url(up.png) no-repeat;
    width: 10px;
    height: 5px;
    margin: 2px;
}

.moouiSpinnerDown {
    cursor: pointer;
    background: url(down.png) no-repeat;
    width: 10px;
    height: 5px;
    line-height: 5px;
    margin: 2px;
}

#statistics .moouiProgressBar {
    margin-top: 10px;
    background: #1c2431;
    -moz-border-radius:5px; /*ff only setting*/
	-webkit-border-radius: 5px;
}

#statistics .moouiProgressBar span {
    line-height: 20px;
}

#addTorrent select {
    width: 520px;
    height: 95px;
    margin: 10px;
    background: #99acc3;
    border:1px solid #23344b;
    color: Black;
}

#addTorrent #addButtons {
    width: 250px;
    margin: 0 auto;
}

#addTorrent #tabs .moouiTabPage {
    height: 200px;
}

#addTorrent .filesTable {
    margin: 10px;
    width: 510px;
    height: 180px;
    color: Black;
    background: #99acc3;
    border:1px solid #23344b;
    overflow: auto;
}

#addTorrent .filesTable .fileIcon {
    width: 20px;
    background: url(mime_icons/unknown.png) 2px 2px no-repeat;
}

#addTorrent .filesTable .fileSelect {
    width: 20px;
}

#addTorrent .filesTable .fileName {
    width: 410px;
}

#addTorrent .filesTable .fileSize {
    width: 60px;
}

#addTorrent .filesTable .image_jpeg {
    background-image: url(mime_icons/image.png);
}

#addTorrent #buttons {
    margin-top: 10px;
    float: right;
}

#addTorrent form {
    margin-top: 20px;
}

#addTorrent form label {
    width: 100px;
}

#addTorrent form label.fluid {
    width: auto;
}

#addTorrent form button {
    position: relative;
    top: 40px;
    float: right;
}

#urlInput {
    margin-top: 5px;
    margin-bottom: 10px;
    width: 225px;
}

#createTorrent h3 {
    margin-bottom: 0px;
}

#createTorrentTabs .moouiTabPage {
    height: 110px;
}

#createTorrentTabs textarea {
    width: 360px;
    height: 95px;
    margin: 0;
}

#fileButton #folderButton {
    margin: 0 auto 10px auto;
    text-align: center;
}

#createTorrentFiles {
    height: 90px;
    width: 370px;
    margin: 10px;
    border:1px solid #23344b;
    background: #99acc3;
}

#createTorrentTrackersTable {
    margin: 5px;
    float: left;
    width: 280px;
    height: 95px;
    background: #99acc3;
    border:1px solid #23344b;
    color: Black;
}

#createTorrentTrackersTable thead tr {
    background: #304663 url(/static/images/simple_bg_flipped.jpg) repeat-x;
    color: White;
}

#createTorrentTabs .tier {
    width: 70px;
}

#createTorrentTabs .tracker {
    width: 230px;
}

#createTorrentTrackersButtons {
    float: right;
    width: 80px;
}

#createTorrentTrackersButtons button {
    width: 70px;
}

#createButtons {
    float: right;
}
