<h3>Files</h3>
<table id="createTorrentFiles">
</table>
<div id="browse">
    <button class="file">File</button>
    <button class="folder">Folder</button>
</div>
<form>
    <div id="tabs" class="moouiTabs"></div>
</form>
<div id="buttons">
    <button id="cancel">Cancel</button>
    <button id="save">Save</button>
</div>
