/*

-----------------------------------------------------------

Theme Name: Simple
Theme URI: http://deluge-torrent.org
Description: Deluge Theme
Version: 1.0
-----------------------------------------------------------

*/



BODY {
 background: #304663 url(images/simple_bg.jpg) repeat-x;
 font-family: trebuchet ms;
 font-size: 10pt;
 margin: 0;
}

/* GENERIC STYLES */

a img {border: 0px}
hr {color: #627082; margin: 15px 0 15px 0;}


/* STRUCTURE */

#page {
 min-width: 800px;
 margin-left: auto;
 margin-right: auto;
}

#main_content {
	background:url(images/simple_line.jpg) repeat-x;
}

#simple_logo {
  background:url(images/simple_logo.jpg) no-repeat;
}

#main {
	padding-top: 20px;
	padding-left: 20px;
	color: #fff;
}

#main form table {
	border: #2a425c 1px solid;
}

#main form table tr  {
  border: 0px;
}

#main form table tr {
 background: #1f3044;
 font-size: 16px;
 border: 0px;
 white-space: nowrap;
}

#main form table tr td{
 border: 0px;
 color: #fff;
 font-size: 12px;
 white-space: nowrap;
}

#main form table tr a {
 color: #8fa6c3;
 font-size: 16px;
 white-space: nowrap;
}

#main form table tr a, a:active, a:visited { color: #8fa6c3; text-decoration: none; }
#main form table tr  a:hover {color: #fff; text-decoration: underline;}

#main form table tr td a {
 color: #fff;
 font-size: 12px;
white-space: nowrap;
}
#main form table tr td a, a:active, a:visited { color: #fff; text-decoration: none;}
#main form table tr td  a:hover {color: #fff; text-decoration: underline;}

#main a {
 color: #fff;
 font-size: 12px;
}
#main a, a:active, a:visited { color: #fff; text-decoration: none;}
#main  a:hover {color: #fff; text-decoration: underline;}

/*info {
 text-align: right;
 padding: 0 50px 0 0;
 color: #8fa6c3;
 font-size: 16px;
 letter-spacing: 4px;
 font-weight: bold;
}*/

.title {
 color: #8fa6c3;
 font-size: 16px;
 letter-spacing: 2px;
 font-weight: bold;
}


.title a, a:active, a:visited { color: #dce4ee; text-decoration: none;}
.title a:hover {color: #fff; text-decoration: underline;}



#button {
 border:1px solid #23344b;
 background: #99acc3;
 color: #000;
 font-family:verdana, arial, helvetica, sans-serif;
 font-size:10px;
 margin-top:5px;
}



INPUT{
 border:1px solid #23344b;
 background: #99acc3;
 color: #000;
}



TEXTAREA{
 border:1px solid #23344b;
 background: #99acc3;
 width:480px;
}


.footertext a { color: #c0c0c0; text-decoration:none;}
.footertext a:visited { color: #c0c0c0; text-decoration:none;}
.footertext a:active { color: #c0c0c0; text-decoration:none;}
.footertext  a:hover {color: #fff; text-decoration: underline;}


.footertext {
 text-align: center;
 padding: 60px 0 0 0;
 font-size: 8pt;
 left: -100px;
 font-family: trebuchet MS;
 color: #fff;
 position: relative;
}



.clearfix:after {
    content: ".";
    display: block;
    height: 0;
    clear: both;
    visibility: hidden;
}

div.progress_bar{
	background-color:#4573a5;
	/*color:blue;*/
	-moz-border-radius:5px; /*ff only setting*/
	}

div.progress_bar_outer { /*used in table-view*/
	width:150px;
}

td.progress_bar {
	white-space: nowrap;
}

td.info_label {
	 font-weight: bold;
}

td {
 font-size: 10pt;
 color: #d1dae5;
 white-space: nowrap;
}

tr {
 font-size: 10pt;
 color: #d1dae5;
}

div.panel {
	padding:10px;
	width:750px;
	background-color: #37506f;
	-moz-border-radius:10px; /*ff-only!*/
	margin-top:10px;
	margin-bottom:10px;
}


/*New styles:*/

div.deluge_button {
	display:inline;
}
form.deluge_button {
	display:inline;
}
button.deluge_button  {
	background-color: #37506f;
	border:1px solid #68a;

	background: #99acc3;
	color: #000;
	vertical-align:middle;
	-moz-border-radius:7px;
}
button.deluge_button:hover {
	background-color:#68a;
}
div.error {
	background-color:#FFFFFF;
	color:#AA0000;
	font-weight:bold;
	-moz-border-radius:10px;
	width:400px;
	margin-bottom:20px;
	padding:10px;

}

/*tr.torrent_table:hover {
	background-color:#68a;
}*/

tr.torrent_table_selected {
	background-color:#900;
}


img.button {
		margin-bottom:0px;
		padding:0px;
		position:relative;
		top:2px;
}

body.inner {
		background:none;
}


form.pause_resume {
	margin:0;
	padding:0;
	border:0;
}

th {
	 background: #1f3044;
	 font-size: 14px;
	 border: 0px;
	 white-space: nowrap;
}

#torrent_table {
	border: #2a425c 1px solid;
}

#config_chooser {
	float: left;
	width:150px;
	text-align:left;
	height:60%;
}

#config_chooser ul {
	list-style-type: none;
}

#config_chooser li:hover {
	background-color:#68a;
}

#config_chooser li.selected {
	background-color:#900;
}

#config_panel {
		height:60%;
}
#config_panel th {
		font-size: 12px;
		text-align:right;
		color:#FFFFFF;
}

#config_panel table {
		background-color:none;
}

h3 {
		background-color:#68a;
		text-align:left;
}

/* Hides from IE-mac \*/

* html .clearfix {height: 1%;}

.clearfix {display: block;}

/* End hide from IE-mac */
