html, body {
    padding: 0;
    margin: 0;
}

body {
    font-family: trebuchet ms;
	font-size: 0.7em;
	color: White;
}

/* forms */

form label {
	width: 150px;
	line-height: 20px;
	float: left;
}

div.fileInputs {
    position: relative;
}

div.fakeFile {
    position: absolute;
    top: 0px;
    left: 0px;
    z-index: 1;
}

div.fakeFile input {
    width: 305px;
}

form input.file {
    position: absolute;
    top: 0px;
    left: 0px;
    text-align: left;
    z-index: 2;
}

form input, form select {
	float: left;
	margin-right: 5px;
	border:1px solid #23344b;
	background: #99acc3;
	color: #000;
	line-height: 20px;
}

form torrentFileDiv {
    position: relative;
    opacity: 0;
}

form textarea {
	border:1px solid #23344b;
	background: #99acc3;
	width:480px;
}

form .disabled {
	color: Gray;
}

form button  {
	background-color: #37506f;
	border:1px solid #68a;
	cursor: pointer;
	background: #99acc3;
	color: #000;
	vertical-align:middle;
	-moz-border-radius:7px;
	-webkit-border-radius:7px;
	margin: 2px;
}

form button:hover {
	background-color:#68a;
}

form br {
	clear: left;
}
