<form>
	<fieldset>
		<legend>$_('Incoming Ports')</legend>
		<label>
			<input type="checkbox" name="random_port" />
			$_('Use Random Ports')
		</label><br/>
		
		<label class="fluid">
			$_('From'):
		</label>
		<input type="text" name="start_listen_port" rel="spinner" />
		<label class="fluid">
			$_('To'):
		</label>
		<input type="text" name="end_listen_port" rel="spinner" /><br/>
	</fieldset>
	
	<fieldset>
		<legend>$_('Outgoing Ports')</legend>
		<label>
			<input type="checkbox" name="random_outgoing_ports" />
			$_('Use Random Ports')
		</label><br/>
		
		<label class="fluid">
			$_('From'):
		</label>
		<input type="text" name="start_outgoing_port" rel="spinner" />
		<label class="fluid">
			$_('To'):
		</label>
		<input type="text" name="end_outgoing_port" rel="spinner" /><br/>
	</fieldset>
	
	<fieldset>
		<legend>$_('TOS')</legend>
		<label>$_('Peer TOS Byte'):</label>
		<input type="text" name="peer_tos" />
	</fieldset>
	
	<fieldset>
		<legend>$_('Network Extras')</legend>
		<label>
			<input type="checkbox" name="upnp" />
			$_('UPnP')
		</label>
		<label>
			<input type="checkbox" name="natpmp" />
			$_('NAT-PMP')
		</label>
		<label>
			<input type="checkbox" name="dht" />
			$_('Enable Mainline DHT')
		</label>
		<label>
			<input type="checkbox" name="utpex" />
			$_('Peer Exchange')
		</label>
		<label>
			<input type="checkbox" name="lsd" />
			$_('LSD')
		</label>
	</fieldset>
	<fieldset id="encryption">
		<legend>$_('Encryption')</legend>
		
		<label>$_('Inbound'):</label>
		<select name="enc_in_policy">
			<option value="0">$_('Forced')</option>
			<option value="1">$_('Enabled')</option>
			<option value="2">$_('Disabled')</option>
		</select>
		
		<label>$_('Outbound'):</label>
		<select name="enc_out_policy">
			<option value="0">$_('Forced')</option>
			<option value="1">$_('Enabled')</option>
			<option value="2">$_('Disabled')</option>
		</select><br/>
		
		<label>$_('Level'):</label>
		<select name="enc_level">
			<option value="0">$_('Handshake')</option>
			<option value="1">$_('Full Stream')</option>
			<option value="2">$_('Either')</option>
		</select>
		
		<label class="fluid">
			<input type="checkbox" name="enc_prefer_rc4" />
			$_('Encrypt entire stream')
		</label>
	</fieldset>
</form>
