_('# Of Files')
_('... and delete All files')
_('... and delete Downloaded files')
_('... and delete Torrent file')
_('About')
_('Active time')
_('Add Torrent')
_('Add Torrents')
_('Add torrent')
_('Address')
_('Admin')
_('Auto Managed')
_('Auto refresh:')
_('Ava')
_('Availability')
_('Bottom')
_('Cancel')
_('Clear')
_('Client')
_('Config')
_('Connect')
_('Connect to Daemon')
_('Connected to')
_('Connection Limit')
_('Connections')
_('Create Torrent')
_('D/L Speed Limit')
_('DHT Nodes')
_('Delete .torrent file')
_('Delete downloaded files.')
_('Deluge : Torrent List')
_('Deluge Login')
_('Details')
_('Disable')
_('Disk Space')
_('Do not download')
_('Down')
_('Down Speed')
_('Download')
_('Downloaded')
_('ETA')
_('Edit Trackers')
_('Enable')
_('Error')
_('Eta')
_('False')
_('File')
_('Files')
_('Filter on a keyword')
_('Force Recheck')
_('From Session')
_('From Url')
_('General')
_('High priority')
_('Highest priority')
_('Keyword')
_('Label')
_('Label torrent')
_('Login')
_('Logout')
_('Move')
_('Move Storage')
_('Move torrent')
_('Name')
_('Next Announce')
_('No Incoming Connections')
_('No Label')
_('None')
_('Normal priority')
_('Not Connected to a daemon')
_('Off')
_('Ok')
_('Options')
_('Other')
_('Password')
_('Password is invalid,try again')
_('Pause')
_('Pause all')
_('Peers')
_('Pieces')
_('Private')
_('Progress')
_('Queue')
_('Queue Position')
_('Ratio')
_('Refresh page every:')
_('Remove')
_('Remove Torrent')
_('Remove torrent')
_('Restart')
_('Resume')
_('Resume all')
_('Save')
_('Search')
_('Seed rank')
_('Seeders')
_('Seeding time')
_('Select All')
_('Set')
_('Set Timeout')
_('Share Ratio')
_('Size')
_('Speed')
_('Start')
_('Statistics')
_('Stop')
_('Submit')
_('Top')
_('Torrent list')
_('Total Size')
_('Tracker')
_('Tracker Status')
_('True')
_('U/L Speed Limit')
_('Unlimited')
_('Up')
_('Up Speed')
_('Update')
_('Update Tracker')
_('Upload')
_('Upload Slot Limit')
_('Uploaded')
_('seconds')
