.roar-body
{
	position:				absolute;
	font:					12px/14px "Lucida Grande", Arial, Helvetica, Verdana, sans-serif;
	color:					#fff;
	text-align:				left;
	z-index:				999;
}

.roar
{
	position:				absolute;
	width:					300px;
	cursor:					pointer;
}
.roar-bg
{
	position:				absolute;
	z-index:				1000;
	width:					100%;
	height:					100%;
	left:					0;
	top:					0;
	background-color:		#000;
	-moz-border-radius:		10px;
	-webkit-border-radius:	5px;
	-webkit-box-shadow:		0 0 5px rgba(0, 0, 0, 0.5);
}
.roar-body-ugly .roar
{
	background-color:		#333;
}
.roar-body-ugly .roar-bg
{
	display:				none;
}
.roar h3
{
	position:				relative;
	padding:				15px 10px 0;
	margin:					0;
	border:					0;
	font-size:				13px;
	color:					#fff;
	z-index:				1002;
}
.roar p
{
	position:				relative;
	padding:				10px 10px 15px;
	margin:					0;
	font-size:				12px;
	color:					#fff;
	z-index:				1002;
}