$def with (title, active_tab="NONE")
<html>
    <head>
        <title>Deluge:$title</title>
        <link rel="icon" href="$base/static/images/deluge-icon.png" type="image/png" />
	    <link rel="shortcut icon" href="$base/static/images/deluge-icon.png" type="image/png" />
        <link rel="stylesheet" type="text/css" href="$base/static/simple_site_style.css" />
        <!--javascript is only used for plugins in the classic-ui
        classic ui should function without  js.
        -->
        <script language="javascript" src="$base/static/deluge.js"></script>
        <script language="javascript" src="$base/gettext.js"></script>
        <script language="javascript" src="$base/static/mootools-1.2-core.js"></script>
        <script language="javascript" src="$base/static/mootools-1.2-more.js"></script>
        <script language="javascript" src="$base/static/mooui.js"></script>
        $for js in include_javascript:
            <script language="javascript" src="$base$js"></script>
    </head>
    <body>
        <div id="page">
<!---
            <a href='$base/home'>
                <div id="simple_logo">

                </div>
            </a>
-->
   <img src="$base/static/images/deluge32.png" style="position:absolute;top:5px;"\>
    &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;
    &nbsp;&nbsp;

<div style="padding-left:50px">
    <span class="title">Deluge</span>
    <a
    $if 'home' == active_tab:
         class="tab_button_active"
    $else:
        class="tab_button"
    href='$base/home'>Home</a>

<!--ADMIN-TABS-->
$for id, title, url in admin_pages:
    <a
    $if id == active_tab:
         class="tab_button_active"
    $else:
        class="tab_button"
    href='$base$url'>$title</a>
<a href="http://dev.deluge-torrent.org/wiki/Faq">Faq</a>
$if is_auto_refreshed:
    <div style="position:absolute;right:20px;top:10px;width:120px;height:20px;overflow:hidden">
    $:render.part_auto_refresh()
    </div>
</div>


            <div id="main_content">
                <div id="main">
                    <center>
