$def with (torrent)
<form  method="POST" action="$base/torrent/trackers/$torrent.id">
<input type="hidden" name="redir" value="$self_url()">
<table width="100%">
$altrow(True)
<tr class="head">
    <th width="10px">#</th>
    <th width="100%">$_("Tracker")</th>
</tr>
$for tracker in torrent.trackers:
    <tr class="$altrow()" >
    <td><input type="text" name="tier" value="$tracker['tier']" style="width:20px"/>
    </td>
        <td><input type="text" name="url" value="$tracker['url']" style="width:400px"//></td>
    </tr>
<tr class="$altrow()" >
<td><input type="text" name="tier" value="$(tracker['tier'] + 1)" style="width:20px"/>
</td>
    <td><input type="text" name="url" value="" style="width:400px"//></td>
</tr>

</table>
<input type="submit" value="$_('Update')">
</form>
