<dl>
	<dt class="torrent_name">$_('Name'):</dt><dd class="torrent_name"/>
	<dt class="hash">$_('Hash'):</dt><dd class="hash"/>
	<dt class="path">$_('Path'):</dt><dd class="path"/>
	<dt class="size">$_('Total Size'):</dt><dd class="size"/>
	<dt class="files">$_('# of files'):</dt><dd class="files"/>
	<dt class="status">$_('Status'):</dt><dd class="status"/>
	<dt class="tracker">$_('Tracker'):</dt><dd class="tracker"/>
</dl>
<br style="clear: both;" />
