<h3>$_('Torrents')</h3>
<select size="8"></select>
<div id="addButtons">
    <button class="file">$_('File')</button>
    <button class="url">$_('Url')</button>
    <button class="infohash">$_('Infohash')</button>
    <button class="remove">$_('Remove')</button>
</div>
<div id="tabs" class="moouiTabs"></div>
<div id="buttons">
    <button class="cancel">$_('Cancel')</button>
    <button class="add">$_('Add')</button>
</div>
